import skimage.io as io
import skimage.color as color
import matplotlib.pyplot as plt
from skimage.color import rgb2gray
import numpy as np
import math
import sys
from skimage import feature
import random
import time


def seedfill_rec(im, seed_row, seed_col, fill_color, bckg):
    row, col, foo = image.shape
    if seed_row>=0 and seed_row<row and seed_col>=0 and seed_col<col:
        if np.array_equal(image[seed_row,seed_col,:],bckg):
            image[seed_row,seed_col]=fill_color
            seedfill_rec(image, seed_row-1, seed_col, fill_color, bckg)
            seedfill_rec(image, seed_row+1, seed_col, fill_color, bckg)
            seedfill_rec(image, seed_row, seed_col-1, fill_color, bckg)
            seedfill_rec(image, seed_row, seed_col+1, fill_color, bckg)

def seedfill(im, seed_row, seed_col, fill_color,bckg):
    """
    im: The image on which to perform the seedfill algorithm
    seed_row and seed_col: position of the seed pixel
    fill_color: Color for the fill
    bckg: Color of the background, to be filled
    Returns: Nothing
    Behavior: Modifies image by performing seedfill
    """
    size=0  # keep track of patch size
    n_row, n_col, foo = im.shape
    front=[(seed_row,seed_col)]  # initial front
    
    while len(front)>0:
        r, c = front.pop(0)  # remove 1st element of front
        if np.array_equal(im[r, c,:],bckg):
            im[r, c]=fill_color  # color pixel
            size+=1
            # look at all neighbors
            for i in range(max(0,r-1), min(n_row,r+2)):
                for j in range(max(0,c-1),min(n_col,c+2)):
                    # if background, add to front
                    if np.array_equal(im[i,j,:], bckg) and\
                       (i,j) not in front:
                        front.append((i,j))
    return size



def seedfill_with_animation(im, seed_row, seed_col, fill_color, background):
    """
    Args:
    im: The image on which to perform the seedfill algorithm
    seed_row and seed_col: position of the seed pixel
    fill_color: Color for the fill
    background: Color of the background, to be filled
    Returns: Nothing
    Behavior: Modifies image by performing seedfill
    """
    size=0  # keep track of patch size
    n_row, n_col, foo = im.shape
    front=[(seed_row,seed_col)]  # initial front
    
    while len(front)>0:
        r, c = front.pop(0)
        if np.array_equal(im[r, c,:],background):
            im[r, c]=fill_color
            if size%1000==0:
                plt.clf()
                plt.imshow(im)
                plt.show()
                plt.pause(0.0001)

            size+=1
            for new_r in range(max(0,r-1), min(n_row,r+2)):
                for new_c in range(max(0,c-1), min(n_col,c+2)):
                    if np.array_equal(im[new_r,new_c,:],background):
                        if (new_r, new_c) not in front:
                            front.append((new_r, new_c))
            
    return size

def blur(image, filter_size):
    n_row, n_col, colors = image.shape
    blurred_image = np.zeros( (n_row, n_col, 3), dtype=np.uint8)
    for i in range(0,n_row):
        for j in range(0,n_col):
            left=max(0,i-filter_size)
            right=min(i+filter_size,n_row)
            bottom=max(0,j-filter_size)
            top=min(n_col,j+filter_size)
            blurred_image[i,j] = image[left:right,bottom:top,:].mean(axis=(0,1))         
    return blurred_image

def detect_edges(im, min_gradient=50):
    """
    Args:
    im: The image on which to detect edges
    min_gradient: The minimum gradient value 
                  for a pixel to be called an edge
    Returns: An new image with edge pixels set to white, 
                      and everything else set to black
    """    
    n_row, n_col, colors = image.shape

    # create a empty empty of the same size as the original
    edge_image = np.zeros( (n_row,n_col,3), dtype=np.uint8)
    
    for i in range(1,n_row-1):  # avoid the first/last row
        for j in range(1,n_col-1):  # and first/last col
            grad=[0,0,0]
            for c in range(3):  # for each color 
                Lx = float(im[i-1,j,c])-float(im[i+1,j,c])
                Ly = float(im[i,j-1,c])-float(im[i,j+1,c])
                grad[c] = math.sqrt(Lx**2+Ly**2)
            norm = math.sqrt(grad[0]**2 + grad[1]**2 \
                             + grad[2]**2)
            if (norm > min_gradient):
                edge_image[i,j] = (255,255,255)
    return edge_image


filename="malaria2"
#filename="car"

image = io.imread(filename+".jpg")

edge = detect_edges(image, 60)

io.imsave(filename+"_edge.jpg",edge)
plt.imshow(edge)
plt.show()
plt.pause(3)

min_cell_size=100  # based on prior knowledge
max_cell_size=300  # based on prior knowledge
n_cells=0
# look for a black pixel to seed the filling
for i in range(image.shape[0]):
    for j in range(image.shape[1]):
        if np.array_equal(edge[i,j,:],(0,0,0)):
            rand_color = (random.randrange(255),
                          random.randrange(255),
                          random.randrange(255))
            size=seedfill_with_animation(edge, i ,j, 
                                         rand_color, (0,0,0) )
            if size>= min_cell_size and size<max_cell_size:
                n_cells+=1
print("Number of cells:",n_cells)
            
io.imsave(filename+"_edge_fill.jpg",edge)
plt.imshow(edge)
plt.show()