import skimage.io as io
import skimage.color as color
import matplotlib.pyplot as plt
from skimage.color import rgb2gray
import numpy as np
import math
import sys
from skimage import feature
import random
import time

def widen(image, color_to_widen, width):
    row, col, foo = image.shape
    widened_image=image.copy()
    for r in range(row):
        for c in range(col):
            if np.array_equal(image[r,c,:],color_to_widen):
                for new_r in range(max(0,r-width), min(row,r+width+1)):
                    for new_c in range(max(0,c-width), min(col,c+width+1)):  
                        widened_image[new_r, new_c]=color_to_widen
    return widened_image

def seedfill_recursive(image, seed_row, seed_col, fill_color, background_color, depth):
    row, col, foo = image.shape
    if seed_row>=0 and seed_row<row and seed_col>=0 and seed_col<col:
        if np.array_equal(image[seed_row,seed_col,:],background_color):
            image[seed_row,seed_col]=fill_color
            seedfill_recursive(image, seed_row-1, seed_col, fill_color, background_color,depth+1)
            seedfill_recursive(image, seed_row+1, seed_col, fill_color, background_color,depth+1)
            seedfill_recursive(image, seed_row, seed_col-1, fill_color, background_color,depth+1)
            seedfill_recursive(image, seed_row, seed_col+1, fill_color, background_color,depth+1)


def seedfill(image, seed_row, seed_col, fill_color, background):
    """
    Args:
    image: The image on which to perform the seedfill algorithm
    seed_row and seed_col: position of the seed pixel
    fill_color: Color for the fill
    background: Color of the background, to be filled
    Returns: Nothing
    Behavior: Modified image by performing seedfill from seed_row,seed_col
    """
    size=0;
    n_row, n_col, foo = image.shape
    front=[(seed_row,seed_col)]
    while len(front)>0:
        r, c = front.pop(0)
        if np.array_equal(image[r, c,:],background):
            image[r, c]=fill_color
            if size%1000==0:
                plt.clf()
                plt.imshow(image)
                plt.show()
                plt.pause(0.0001)

            size+=1
            for new_row in range(max(0,r-1), min(n_row,r+2)):
                for new_col in range(max(0,c-1), min(n_col,c+2)):
                    if np.array_equal(image[new_row,new_col,:],background):
                        if (new_row, new_col) not in front:
                            front.append((new_row, new_col))
            
    return size

def blur(image, filter_size):
    n_row, n_col, colors = image.shape
    blurred_image = np.zeros( (n_row, n_col, 3), dtype=np.uint8)
    for i in range(0,n_row):
        for j in range(0,n_col):
            left=max(0,i-filter_size)
            right=min(i+filter_size,n_row)
            bottom=max(0,j-filter_size)
            top=min(n_col,j+filter_size)
            blurred_image[i,j] = image[left:right,bottom:top,:].mean(axis=(0,1))         
    return blurred_image

def detect_edges(image, min_gradient):
    """
    Args:
    image: The image on which to perform the seedfill algorithm
    min_gradient: The minimum gradient value to call a pixel an edge
    Returns: An image with edge pixels set to white, 
                      and everything else set to black
    """    
    n_row, n_col, colors = image.shape

    # create a new, empty empty of the same size as the original
    edge_image = np.zeros( (n_row, n_col, 3), dtype=np.uint8)
    
    for i in range(1,n_row-1):  # avoid the first and last row
        for j in range(1,n_col-1):  # and first/last col
            gradient=[0,0,0]
            for c in range(3):
                Lx = float(image[i-1,j,c])-float(image[i+1,j,c])
                Ly = float(image[i,j-1,c])-float(image[i,j+1,c])
                gradient[c] = math.sqrt(Lx**2+Ly**2)
            norm = math.sqrt(gradient[0]**2 + gradient[1]**2 + gradient[2]**2)
            if (norm > min_gradient):
                edge_image[i,j] = (255,255,255)
                
    return edge_image


filename="malaria2"
#filename="car"

image = io.imread(filename+".jpg")


edge = detect_edges(image, 60)

io.imsave(filename+"_edge.jpg",edge)
plt.imshow(edge)
plt.show()
plt.pause(3)

patch_sizes=[]
# look for a black pixel to seed the filling
for row in range(image.shape[0]):
    for col in range(image.shape[1]):
        if np.array_equal(edge[row,col,:],(0,0,0)):
            rand_color = (random.randrange(255),random.randrange(255),random.randrange(255))
            size=seedfill(edge, row, col, rand_color, (0,0,0) )
            patch_sizes.append(size)
            
io.imsave(filename+"_edge_fill.jpg",edge)
plt.imshow(edge)
plt.show()

plt.clf()
print(patch_sizes)
plt.hist(patch_sizes,bins=100, range=(0,300))
plt.xlabel("Patch size")
plt.ylabel("Frequency")
plt.show()
plt.savefig("histogram_patch_size.png")
